/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.junit;

import java.io.StringReader;
import java.io.StringWriter;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.util.Calendar;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.ws.jaxme.JMMarshaller;
import org.apache.ws.jaxme.JMUnmarshaller;
import org.apache.ws.jaxme.JMXmlSerializer;
import org.apache.ws.jaxme.XMLWriter;
import org.apache.ws.jaxme.impl.JAXBContextImpl;
import org.apache.ws.jaxme.impl.JMMarshallerImpl;
import org.apache.ws.jaxme.impl.XMLWriterImpl;
import org.apache.ws.jaxme.test.misc.types.AllSimpleTypesType;
import org.apache.ws.jaxme.test.misc.types.AllTypesElement;
import org.apache.ws.jaxme.test.misc.types.impl.AllSimpleTypesTypeHandler;
import org.apache.ws.jaxme.test.misc.types.impl.AllSimpleTypesTypeImpl;
import org.apache.ws.jaxme.test.misc.types.impl.AllSimpleTypesTypeSerializer;
import org.apache.ws.jaxme.test.misc.types.impl.AllTypesElementImpl;
import org.apache.ws.jaxme.util.Duration;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;


/**
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 * @version $Id: MarshallerTest.java,v 1.3 2003/11/03 20:50:50 jochen Exp $
 */
public class MarshallerTest extends BaseTestCase {
  private JAXBContextImpl factory;

  public MarshallerTest(String arg) { super(arg); }

  public void setUp() throws JAXBException {
    factory = (JAXBContextImpl) JAXBContext.newInstance("org.apache.ws.jaxme.test.misc.types");
  }

  public JAXBContextImpl getFactory() {
    return factory;
  }

  public Calendar getDateTime() {
    Calendar cal = Calendar.getInstance();
    cal.set(Calendar.DAY_OF_MONTH, 22);
    cal.set(Calendar.MONTH, 11);
    cal.set(Calendar.YEAR, 2002);
    cal.set(Calendar.HOUR_OF_DAY, 16);
    cal.set(Calendar.MINUTE, 43);
    cal.set(Calendar.SECOND, 37);
    return cal;
  }
  public Calendar getTime() {
    Calendar cal = Calendar.getInstance();
    cal.set(Calendar.HOUR_OF_DAY, 16);
    cal.set(Calendar.MINUTE, 43);
    cal.set(Calendar.SECOND, 37);
    return cal;
  }
  public Calendar getDate() {
    Calendar cal = Calendar.getInstance();
    cal.set(Calendar.DAY_OF_MONTH, 22);
    cal.set(Calendar.MONTH, 11);
    cal.set(Calendar.YEAR, 2002);
    return cal;
  }
  public Duration getDuration() {
    Duration d = new Duration(1, 2, 3, 4, 5, 6.7);
    return d;
  }
  public byte[] getHexBytes() {
    return new byte[]{1, 17, 35, 78, 115, -99, -69, -1};
  }

  public AllSimpleTypesType getAllSimpleTypesElement() {
    AllSimpleTypesType element = new AllSimpleTypesTypeImpl();
    element.setStringElem("This is a string with german Umlauts: \u00e4\u00f6\u00fc\u00df\u00c4\u00d6\u00dc");
    element.setDoubleElem(23979782937923.2398798);
    element.setFloatElem(-34768.3486787f);
    element.setIntElem(-2139879);
    element.setLongElem(1290380128309182303l);
    element.setShortElem((short) 23878);
    element.setDateElem(getDate());
    element.setDateTimeElem(getDateTime());
    element.setTimeElem(getTime());
    element.setDurationElem(getDuration());
    element.setHexBinaryElem(getHexBytes());
    element.setBase64BinaryElem(getHexBytes());
    element.setNmTokenElem("a-name-token"); 
    List nmTokens = element.getNmTokensElem();
    nmTokens.add("a-name-token");
    nmTokens.add("another:name-token");
    element.setDecimalElem(new BigDecimal("6.023e23"));
    element.setIntegerElem(new BigDecimal("-6023e20").toBigInteger());
    element.setNonNegativeIntegerElem(new BigInteger("101"));
    element.setPositiveIntegerElem(new BigDecimal("+6023e20").toBigInteger());
    element.setNonPositiveIntegerElem(new BigInteger("-21714"));
    element.setNegativeIntegerElem(new BigInteger("-21714"));
    return element;
  }

  public AllTypesElement getAllTypesElement() throws JAXBException {
    AllTypesElement element = (AllTypesElement) getFactory().getElement(AllTypesElement.class);
    element.setAllSimpleTypesElement(getAllSimpleTypesElement());
    List list = element.getListTypeElement();
    list.add(new BigInteger("7"));
    list.add(new BigInteger("-3"));
    list.add(new BigInteger("0"));
    element.setUnionTypeElement(getDateTime());
    return element;
  }

  public String getAllSimpleTypesElementString() {
    AllTypesElementImpl elem = new AllTypesElementImpl();
    String uri = elem.getQName().getNamespaceURI();
    return
      "<ex:AllSimpleTypesElement xmlns:ex=\"" + uri + "\">" +
      "<ex:StringElem>This is a string with german Umlauts: &#228;&#246;&#252;&#223;&#196;&#214;&#220;</ex:StringElem>" +
      "<ex:IntElem>-2139879</ex:IntElem>" +
      "<ex:LongElem>1290380128309182303</ex:LongElem>" +
      "<ex:ShortElem>23878</ex:ShortElem>" +
      "<ex:DoubleElem>2.397978293792324E13</ex:DoubleElem>" +
      "<ex:FloatElem>-34768.348</ex:FloatElem>" +
      "<ex:DateTimeElem>" + convertUnicode(DateFormat.getDateTimeInstance().format(getDateTime().getTime())) + "</ex:DateTimeElem>" +
      "<ex:DateElem>" + DateFormat.getDateInstance().format(getDate().getTime()) + "</ex:DateElem>" +
      "<ex:TimeElem>" + convertUnicode(DateFormat.getTimeInstance().format(getTime().getTime())) + "</ex:TimeElem>" +
      "<ex:DurationElem>P1Y2M3DT4H5M" + NumberFormat.getInstance().format(6.7) + "S</ex:DurationElem>" +
      "<ex:HexBinaryElem>0111234E739DBBFF</ex:HexBinaryElem>" +
      "<ex:Base64BinaryElem>AREjTnOdu/8=</ex:Base64BinaryElem>" +
      "<ex:NmTokenElem>a-name-token</ex:NmTokenElem>" +
      "<ex:NmTokensElem>a-name-token another:name-token</ex:NmTokensElem>" +
      "<ex:DecimalElem>602300000000000000000000</ex:DecimalElem>" +
      "<ex:IntegerElem>-602300000000000000000000</ex:IntegerElem>" +
      "<ex:NonNegativeIntegerElem>101</ex:NonNegativeIntegerElem>" +
      "<ex:PositiveIntegerElem>602300000000000000000000</ex:PositiveIntegerElem>" +
      "<ex:NonPositiveIntegerElem>-21714</ex:NonPositiveIntegerElem>" +
      "<ex:NegativeIntegerElem>-21714</ex:NegativeIntegerElem>" +
      "</ex:AllSimpleTypesElement>";
  }

  public String getAllTypesElementString() {
    AllTypesElementImpl elem = new AllTypesElementImpl();
    String uri = elem.getQName().getNamespaceURI();
    return
      "<ex:AllTypesElement xmlns:ex=\"" + uri + "\">\n" +
      "  <ex:AllSimpleTypesElement>\n" +
      "    <ex:StringElem>This is a string with german Umlauts: &#228;&#246;&#252;&#223;&#196;&#214;&#220;</ex:StringElem>\n" +
      "    <ex:IntElem>-2139879</ex:IntElem>\n" +
      "    <ex:LongElem>1290380128309182303</ex:LongElem>\n" +
      "    <ex:ShortElem>23878</ex:ShortElem>\n" +
      "    <ex:DoubleElem>2.397978293792324E13</ex:DoubleElem>\n" +
      "    <ex:FloatElem>-34768.348</ex:FloatElem>\n" +
      "    <ex:DateTimeElem>" + convertUnicode(DateFormat.getDateTimeInstance().format(getDateTime().getTime())) + "</ex:DateTimeElem>\n" +
      "    <ex:DateElem>" + DateFormat.getDateInstance().format(getDate().getTime()) + "</ex:DateElem>\n" +
      "    <ex:TimeElem>" + convertUnicode(DateFormat.getTimeInstance().format(getTime().getTime())) + "</ex:TimeElem>\n" +
      "    <ex:DurationElem>P1Y2M3DT4H5M" + NumberFormat.getInstance().format(6.7) + "S</ex:DurationElem>\n" +
      "    <ex:HexBinaryElem>0111234E739DBBFF</ex:HexBinaryElem>\n" +
      "    <ex:Base64BinaryElem>AREjTnOdu/8=</ex:Base64BinaryElem>\n" +
      "    <ex:NmTokenElem>a-name-token</ex:NmTokenElem>\n" +
      "    <ex:NmTokensElem>a-name-token another:name-token</ex:NmTokensElem>\n" +
      "    <ex:DecimalElem>602300000000000000000000</ex:DecimalElem>\n" +
      "    <ex:IntegerElem>-602300000000000000000000</ex:IntegerElem>\n" +
      "    <ex:NonNegativeIntegerElem>101</ex:NonNegativeIntegerElem>\n" +
      "    <ex:PositiveIntegerElem>602300000000000000000000</ex:PositiveIntegerElem>\n" +
      "    <ex:NonPositiveIntegerElem>-21714</ex:NonPositiveIntegerElem>\n" +
      "    <ex:NegativeIntegerElem>-21714</ex:NegativeIntegerElem>\n" +
      "  </ex:AllSimpleTypesElement>\n" +
      "  <ex:ListTypeElement>7 -3 0</ex:ListTypeElement>\n" +
      "  <ex:UnionTypeElement>" + convertUnicode(DateFormat.getDateTimeInstance().format(getDateTime().getTime())) + "</ex:UnionTypeElement>\n" +
      "</ex:AllTypesElement>";
  }

  public XMLReader getXMLReader() throws ParserConfigurationException, SAXException {
    // Verify whether we can create a SAX Parser; it's better to detect this here
    // than within JAXB
    SAXParserFactory spf = SAXParserFactory.newInstance();
    spf.setNamespaceAware(true);
    spf.setValidating(false);
    SAXParser sp = spf.newSAXParser();
    return sp.getXMLReader();
  }

  public void verifyAllSimpleTypesElement(AllSimpleTypesType pElement) {
    assertStringEquals("This is a string with german Umlauts: \u00e4\u00f6\u00fc\u00df\u00c4\u00d6\u00dc",
                       pElement.getStringElem());
    assertEquals(new Double("2.397978293792324E13").doubleValue(),
                 pElement.getDoubleElem(), 0);
    assertEquals(new Float(-34768.348).floatValue(), pElement.getFloatElem(), 0);
    assertEquals(new Integer(-2139879).intValue(), pElement.getIntElem());
    assertEquals(new Long(1290380128309182303L).longValue(), pElement.getLongElem());
    assertEquals(new Short((short) 23878).shortValue(), pElement.getShortElem());
    String dateTime1 = DateFormat.getDateTimeInstance().format(getDateTime().getTime());
    String dateTime2 = DateFormat.getDateTimeInstance().format(pElement.getDateTimeElem().getTime());
    assertStringEquals(dateTime1, dateTime2);
    String date1 = DateFormat.getDateInstance().format(getDate().getTime());
    String date2 = DateFormat.getDateInstance().format(pElement.getDateElem().getTime());
    assertStringEquals(date1, date2);
    String time1 = DateFormat.getTimeInstance().format(getTime().getTime());
    String time2 = DateFormat.getTimeInstance().format(pElement.getTimeElem().getTime());
    assertStringEquals(time1, time2);
    assertEquals(getDuration(), pElement.getDurationElem());
    assertEquals(getHexBytes(), pElement.getHexBinaryElem());
    assertEquals(getHexBytes(), pElement.getBase64BinaryElem());
    assertEquals(new BigInteger("101"), pElement.getNonNegativeIntegerElem());
  }

  public void verifyAllTypesElement(AllTypesElement pElement) {
    verifyAllSimpleTypesElement(pElement.getAllSimpleTypesElement());
    List list = pElement.getListTypeElement();
    assertEquals(3, list.size());
    assertEquals(new BigInteger("7"), list.get(0));
    assertEquals(new BigInteger("-3"), list.get(1));
    assertEquals(new BigInteger("0"), list.get(2));
    Object object = pElement.getUnionTypeElement();
    String dateTime1 = DateFormat.getDateTimeInstance().format(getDateTime().getTime());
    String dateTime2 = DateFormat.getDateTimeInstance().format(((Calendar) object).getTime());
    assertStringEquals(dateTime1, dateTime2);
  }

  public void testUnmarshalSimpleElements() throws Exception {
    XMLReader xr = getXMLReader();
    AllSimpleTypesTypeHandler unmarshaller = new AllSimpleTypesTypeHandler();
    JMUnmarshaller jmUnmarshaller = (JMUnmarshaller) getFactory().createUnmarshaller();
    unmarshaller.init(jmUnmarshaller);
    xr.setContentHandler(unmarshaller);
    String s = getAllSimpleTypesElementString();
    StringReader sr = new StringReader(s);
    xr.parse(new InputSource(sr));
    AllSimpleTypesType result = (AllSimpleTypesType) unmarshaller.getResult();
    verifyAllSimpleTypesElement(result);
  }

  public void testUnmarshalComplexElements() throws Exception {
    JAXBContext myFactory = getFactory();
    Unmarshaller unmarshaller = myFactory.createUnmarshaller();
    StringReader sr = new StringReader(getAllTypesElementString());
    AllTypesElement result = (AllTypesElement) unmarshaller.unmarshal(new InputSource(sr));
    verifyAllTypesElement(result);
  }

  public void testMarshalSimpleElements() throws Exception {
    AllSimpleTypesTypeSerializer marshaller = new AllSimpleTypesTypeSerializer();
    marshaller.init(getFactory());
    XMLWriter xw = new XMLWriterImpl();
    StringWriter sw = new StringWriter();
    xw.setWriter(sw);
    JMXmlSerializer.Data data = marshaller.getData((JMMarshaller) factory.createMarshaller(), xw);
    AllTypesElementImpl element = new AllTypesElementImpl();
    marshaller.marshal(data, new QName(element.getQName().getNamespaceURI(), "AllSimpleTypesElement"), getAllSimpleTypesElement());
    assertStringEquals(getAllSimpleTypesElementString(), sw.toString());
  }

  public void testMarshalComplexElements() throws Exception {
    JAXBContext myFactory = getFactory();
    JMMarshallerImpl marshaller = (JMMarshallerImpl) myFactory.createMarshaller();
    StringWriter sw = new StringWriter();
    Class c = marshaller.getXMLWriterClass();
    XMLWriter w = (XMLWriter) c.newInstance();
    w.init(marshaller);
    String rawInput = getAllTypesElementString();
    StringBuffer input = new StringBuffer();
    for (int i = 0;  i < rawInput.length();  i++) {
      char ch = rawInput.charAt(i);
      boolean done = false;
      if (ch == '&') {
        if (i+1 < rawInput.length()  &&  rawInput.charAt(i+1) == '#') {
          int j = 2;
          StringBuffer digits = new StringBuffer();
          while (i+j < rawInput.length()  &&  Character.isDigit(rawInput.charAt(i+j))) {
            digits.append(rawInput.charAt(i+j));
            ++j;
          }
          if (digits.length() > 0  &&
              i+j < rawInput.length()  &&  rawInput.charAt(i+j) == ';') {
            char chr = (char) Integer.parseInt(digits.toString());
            if (w.canEncode(chr)) {
              done = true;
              i += j;
              input.append(chr);
            }
          }
        }
      }
      if (!done) {
        input.append(ch);
      }
    }
    marshaller.marshal(getAllTypesElement(), sw);
    assertStringEquals(input.toString(), sw.toString());
  }
  
  private String convertUnicode(String source)
  {
  	StringWriter targetWriter = new StringWriter();
  	char[] sourceChars = source.toCharArray();
  	for (int i = 0; i < sourceChars.length; i++)
  	{
  		char c = sourceChars[i];
			if (c > 31  &&  c < 127) {
				targetWriter.write(c);
			} else {
				targetWriter.write("&#");
				targetWriter.write(Integer.toString(c));
				targetWriter.write(";");
			}
  	}
  	return targetWriter.toString();
  }
}
