/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The name "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package org.apache.ws.jaxme.util;

import java.io.Serializable;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.text.ParseException;


/** <p>Implementation of xs:duration.</p>
 *
 * @author <a href="mailto:joe@ispsoft.de">Jochen Wiedmann</a>
 */
public class Duration implements Serializable, Comparable {
  private final int years, months, days, hours, minutes;
  private final double seconds;

  public Duration(int pYears, int pMonths, int pDays, int pHours, int pMinutes, double pSeconds) {
    years = pYears;
    months = pMonths;
    days = pDays;
    hours = pHours;
    minutes = pMinutes;
    seconds = pSeconds;
  }
  
  /** <p>Returns the number of years.</p>
   */
  public int getYears() {
    return years;
  }

  /** <p>Returns the number of months.</p>
   */
  public int getMonths() {
    return months;
  }

  /** <p>Returns the number of days.</p>
   */
  public int getDays() {
    return days;
  }

  /** <p>Returns the number of hours.</p>
   */
  public int getHours() {
    return hours;
  }

  /** <p>Returns the number of minutes.</p>
   */
  public int getMinutes() {
    return minutes;
  }

  /** <p>Returns the number of seconds.</p>
   */
  public double getSeconds() {
    return seconds;
  }
  /** <p>Returns a string representation of this Duration.</p>
   */
  public String toString() {
    return "P" + getYears() + "Y" + getMonths() + "M" + getDays() + "DT" +
      getHours() + "H" + getMinutes() + "M" +
      NumberFormat.getInstance().format(getSeconds()) + "S";
  }

  /** <p>Converts the given String representation into an instance of
   * Duration.</p>
   * @throws ParseException The String could not be parsed.
   */
  public static Duration valueOf(String pValue) throws ParseException {
    MessageFormat format = new MessageFormat("P{0,number,integer}Y{1,number,integer}M{2,number,integer}DT{3,number,integer}H{4,number,integer}M{5,number}S");
    Object[] numbers = format.parse(pValue);
    return new Duration(((Number) numbers[0]).intValue(),
                         ((Number) numbers[1]).intValue(),
                         ((Number) numbers[2]).intValue(),
                         ((Number) numbers[3]).intValue(),
                         ((Number) numbers[4]).intValue(),
                         ((Number) numbers[5]).doubleValue());
  }

  public boolean equals(Object o) {
    if (o == null  ||  !(o instanceof Duration)) {
      return false;
    }
    return compareTo((Duration) o) == 0;
  }

  public int compareTo(Object o) {
    return compareTo((Duration) o);
  }

  public int compareTo(Duration d) {
    if (years != d.years) { return years - d.years; }
    if (months != d.months) { return months - d.months; }
    if (days != d.days) { return days - d.days; }
    if (hours != d.hours) { return hours - d.hours; }
    if (minutes != d.minutes) { return minutes - d.minutes; }
    if (seconds > d.seconds) {
      return 1;
    } else if (seconds < d.seconds) {
      return -1;
    } else {
      return 0;
    }
  }

  public int hashCode() {
    return years + months + days + hours + minutes + (int) seconds;
  }
}
